import { defineComponent, h, PropType, computed  } from 'vue';
type ISize = 'xs' | 'sm' | 'md' | 'lg';
type IVariant = 'default'| 'primary' | 'success' | 'info' | 'warning' | 'danger' | 'glass' | 'ghost' | 'link';

const buttonProps = {
  onClick: {
    type: Function as PropType<(e: MouseEvent) => any>,
    required: false,
  },
  size: {
      type: String as PropType<ISize>,
      required: false,
  },
  variant: {
      type: String as PropType<IVariant>,
      required: false,
  },
  disabled: {
      type: Boolean,
      required: false,
      default: false
  },
  outline: {
      type: Boolean,
      required: false,
      default: false,
  },
  wide: {
      type: Boolean,
      required: false,
      default: false,
  },
  active: {
      type: Boolean,
      required: false,
      default: false,
  },
  pill: {
      type: Boolean,
      required: false,
  },
  block: {
      type: Boolean,
      required: false,
  },
  circle: {
      type: Boolean,
      required: false,
  },
  square: {
      type: Boolean,
      required: false,
  },
  disableAnimation: {
      type: Boolean,
      required: false,
  },
  label: {
      type: String,
      required: true,
  }
}

export const Component = defineComponent({
    name: 'r-button',
    props: {
      ...buttonProps
    },
    setup(props, { slots }) {
        //     const {
        //       tag,
        //       type,
        //       size,
        //       disabled,
        //       loading,
        //       rounded,
        //       focused,
        //       hovered,
        //       selected,
        //       active,
        //       outlined,
        //       expanded,
        //       inverted,
        //       to
        //     } = toRefs(props);

        const size = computed(() => props.size || 'md');
        const variant = computed(() => props.variant || 'default');

        // create style dynamically based on size 
        const cls = computed(() => {
            return [
                'r-btn',
                {
                    'r-btn-disabled': props.disabled,
                    'r-btn-wide': props.wide,
                    'r-btn-rounded': props.pill,
                    'r-btn-block': props.block,
                    'r-btn-circle': props.circle,
                    'r-btn-square': props.square,
                    'r-btn-no-animation': props.disableAnimation,
                    [`r-btn-active-${variant.value}`]: props.active, 
                    [`r-btn-outline-${variant.value}`]: props.outline,
                    [`r-btn-${size.value}`]: true,
                    [`r-btn-${variant.value}`]: !props.outline,
                }
            ];
        });

        const handleOnClick = async (e: any) => {
            if (typeof props.onClick !== 'function') return;
            try {
                await props.onClick(e);
            } catch (error) {
                throw error;
            }
        };

        return () => h(
            'button',
            {
                class: cls.value,
                onClick: handleOnClick,
                disabled: props.disabled,
            },
            props.label || slots.default?.()
        )
    }
});